document.addEventListener("DOMContentLoaded", () => {
  // Network visualization
  const canvas = document.getElementById("network-canvas");
  const ctx = canvas.getContext("2d");
  let animationId;
  let isAnimating = false;

  function resizeCanvas() {
    canvas.width = canvas.offsetWidth;
    canvas.height = canvas.offsetHeight;
  }

  window.addEventListener("resize", resizeCanvas);
  resizeCanvas();

  // Network node class
  class Node {
    constructor(x, y, radius, color) {
      this.x = x;
      this.y = y;
      this.radius = radius;
      this.color = color;
      this.connections = [];
    }

    draw() {
      ctx.beginPath();
      ctx.arc(this.x, this.y, this.radius, 0, Math.PI * 2);
      ctx.fillStyle = this.color;
      ctx.fill();
      ctx.closePath();
    }
  }

  // Initialize network
  let nodes = [];

  function initializeNetwork() {
    nodes = [];
    const inputDim = parseInt(document.getElementById("input-dim").value);
    const hiddenUnits = parseInt(document.getElementById("hidden-units").value);
    const attentionHeads = parseInt(
      document.getElementById("attention-heads").value,
    );

    // Input layer
    for (let i = 0; i < inputDim; i++) {
      const x = canvas.width * 0.2;
      const y = (canvas.height * (i + 1)) / (inputDim + 1);
      nodes.push(new Node(x, y, 10, "#4dabf7"));
    }

    // Hidden layer
    for (let i = 0; i < hiddenUnits; i++) {
      const x = canvas.width * 0.5;
      const y = (canvas.height * (i + 1)) / (hiddenUnits + 1);
      nodes.push(new Node(x, y, 8, "#2a6bc2"));
    }

    // Output layer
    for (let i = 0; i < attentionHeads; i++) {
      const x = canvas.width * 0.8;
      const y = (canvas.height * (i + 1)) / (attentionHeads + 1);
      nodes.push(new Node(x, y, 10, "#1d4580"));
    }
  }

  function drawConnections() {
    ctx.strokeStyle = "rgba(42, 107, 194, 0.2)";
    ctx.lineWidth = 1;

    for (let i = 0; i < nodes.length; i++) {
      for (let j = i + 1; j < nodes.length; j++) {
        if (Math.abs(nodes[i].x - nodes[j].x) > canvas.width * 0.1) {
          ctx.beginPath();
          ctx.moveTo(nodes[i].x, nodes[i].y);
          ctx.lineTo(nodes[j].x, nodes[j].y);
          ctx.stroke();
        }
      }
    }
  }

  function animate() {
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    drawConnections();
    nodes.forEach((node) => node.draw());

    if (isAnimating) {
      nodes.forEach((node) => {
        node.y += Math.sin(Date.now() / 1000) * 0.5;
      });
      animationId = requestAnimationFrame(animate);
    }
  }

  // Event listeners
  document.getElementById("apply-config").addEventListener("click", () => {
    initializeNetwork();
    animate();
  });

  document.getElementById("animate-btn").addEventListener("click", () => {
    isAnimating = !isAnimating;
    if (isAnimating) {
      animate();
      document.getElementById("animate-btn").textContent = "Stop Animation";
    } else {
      cancelAnimationFrame(animationId);
      document.getElementById("animate-btn").textContent = "Animate Flow";
    }
  });

  document.getElementById("reset-btn").addEventListener("click", () => {
    isAnimating = false;
    cancelAnimationFrame(animationId);
    document.getElementById("animate-btn").textContent = "Animate Flow";
    initializeNetwork();
    animate();
  });

  // Initialize on load
  initializeNetwork();
  animate();
});

document.addEventListener("DOMContentLoaded", function () {
  // Initialize chart
  const ctx = document.getElementById("lossChart").getContext("2d");
  let lossChart = null;

  // Update value displays
  const rangeInputs = document.querySelectorAll('input[type="range"]');
  rangeInputs.forEach((input) => {
    input.addEventListener("input", function () {
      const display = this.nextElementSibling;
      display.textContent = this.value;
    });
  });

  // Train button click handler
  document
    .getElementById("trainButton")
    .addEventListener("click", async function () {
      this.disabled = true;
      this.textContent = "Training...";

      const params = {
        innerUnits: parseInt(document.getElementById("innerUnits").value),
        outerUnits: parseInt(document.getElementById("outerUnits").value),
        learningRate: parseFloat(document.getElementById("learningRate").value),
        epochs: parseInt(document.getElementById("epochs").value),
        modelType: document.getElementById("modelType").value,
      };

      try {
        // Simulate training process
        const epochs = params.epochs;
        const lossData = [];

        if (lossChart) {
          lossChart.destroy();
        }

        lossChart = new Chart(ctx, {
          type: "line",
          data: {
            labels: [],
            datasets: [
              {
                label: "Training Loss",
                data: [],
                borderColor: "#2a6be6",
                tension: 0.4,
              },
            ],
          },
          options: {
            responsive: true,
            scales: {
              y: {
                beginAtZero: true,
              },
            },
          },
        });

        for (let i = 0; i < epochs; i++) {
          // Simulate training iteration
          const loss = Math.exp(-i / epochs) + Math.random() * 0.1;
          lossData.push(loss);

          lossChart.data.labels.push(i + 1);
          lossChart.data.datasets[0].data.push(loss);
          lossChart.update("none");

          await new Promise((resolve) => setTimeout(resolve, 100));
        }

        // Update metrics
        document.getElementById("aucValue").textContent = (
          0.85 +
          Math.random() * 0.1
        ).toFixed(4);
        document.getElementById("ksValue").textContent = (
          0.75 +
          Math.random() * 0.1
        ).toFixed(4);
        document.getElementById("logLossValue").textContent = (
          0.3 +
          Math.random() * 0.1
        ).toFixed(4);
      } catch (error) {
        console.error("Training error:", error);
      } finally {
        this.disabled = false;
        this.textContent = "Train Model";
      }
    });
});

// Simple Chart implementation (since we can't use external libraries)
class Chart {
  constructor(ctx, config) {
    this.ctx = ctx;
    this.config = config;
    this.data = config.data;
    this.canvas = ctx.canvas;
    this.canvas.width = this.canvas.parentNode.offsetWidth;
    this.canvas.height = 400;
  }

  update() {
    this.draw();
  }

  destroy() {
    this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
  }

  draw() {
    const ctx = this.ctx;
    const data = this.data.datasets[0].data;
    const labels = this.data.labels;

    ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);

    if (data.length < 2) return;

    const padding = 40;
    const width = this.canvas.width - 2 * padding;
    const height = this.canvas.height - 2 * padding;

    const xStep = width / (data.length - 1);
    const yMax = Math.max(...data) * 1.1;
    const yScale = height / yMax;

    // Draw axes
    ctx.beginPath();
    ctx.strokeStyle = "#666";
    ctx.moveTo(padding, padding);
    ctx.lineTo(padding, this.canvas.height - padding);
    ctx.lineTo(this.canvas.width - padding, this.canvas.height - padding);
    ctx.stroke();

    // Draw line
    ctx.beginPath();
    ctx.strokeStyle = this.data.datasets[0].borderColor;
    ctx.lineWidth = 2;

    data.forEach((point, i) => {
      const x = padding + i * xStep;
      const y = this.canvas.height - padding - point * yScale;

      if (i === 0) {
        ctx.moveTo(x, y);
      } else {
        ctx.lineTo(x, y);
      }
    });

    ctx.stroke();
  }
}
